package cn.iocoder.yudao.module.bpm.controller.admin.oa;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.cash.BpmOACashCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.cash.BpmOACashRespVO;
import cn.iocoder.yudao.module.bpm.convert.oa.BpmOACashConvert;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOACashDO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOACashItemDO;
import cn.iocoder.yudao.module.bpm.service.oa.BpmOACashService;
import cn.iocoder.yudao.module.system.api.bank.BankApi;
import cn.iocoder.yudao.module.system.api.bank.dto.BankRespDTO;
import cn.iocoder.yudao.module.system.api.dept.DeptApi;
import cn.iocoder.yudao.module.system.api.dept.dto.DeptRespDTO;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.common.util.collection.CollectionUtils.convertSet;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;

/**
 * OA 现金支出 Controller
 *
 * @author 符溶馨

 */
@Tag(name = "管理后台 - OA 现金支出")
@RestController
@RequestMapping("/bpm/oa/cash")
@Validated
public class BpmOACashController {

    @Resource
    private BpmOACashService cashService;

    @Resource
    private DeptApi deptApi;

    @Resource
    private BankApi bankApi;

    @PostMapping("/create")
    @Operation(summary = "创建请求申请")
    public CommonResult<Long> createCash(@Valid @RequestBody BpmOACashCreateReqVO createReqVO) {

        return success(cashService.createCash(getLoginUserId(), createReqVO));
    }

    @GetMapping("/get")
    @Operation(summary = "获得现金支出申请")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    public CommonResult<BpmOACashRespVO> getCash(@RequestParam("id") Long id) {

        BpmOACashDO cash = cashService.getCash(id);

        // 获取明细数据
        List<BpmOACashItemDO> cashItemDOs = cashService.getCashItem(cash.getId());

        //获取部门信息map
        Map<Long, DeptRespDTO> deptMap =  deptApi.getDeptMap(convertSet(cashItemDOs, BpmOACashItemDO::getDeptId));

        // 获取银行卡信息
        BankRespDTO bankRespDTO = null;
        if (cash.getBankId() != null) {
            bankRespDTO = bankApi.getBank(cash.getBankId()).getCheckedData();
        }

        return success(BpmOACashConvert.INSTANCE.convert1(cash, cashItemDOs, deptMap, bankRespDTO));
    }
}
